@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 4413 2009-01-08 17:31:58Z wwalker $"
-__version__   = "$Revision: 4413 $"
-__date__      = "$Date: 2009-01-08 12:31:58 -0500 (Thu, 08 Jan 2009) $"
+__id__        = "$Id: flat_review.py 4340 2008-11-17 23:02:19Z joanied $"
+__version__   = "$Revision: 4340 $"
+__date__      = "$Date: 2008-11-17 18:02:19 -0500 (Mon, 17 Nov 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -859,45 +859,10 @@ class Context:
         else:
             zones = []
 
+        length = text.characterCount
+
         offset = 0
         lastEndOffset = -1
-        upperMax = lowerMax = text.characterCount
-        upperMid = lowerMid = upperMax /2
-        upperMin = lowerMin = 0
-        upperY = lowerY = 0
-        oldMid = 0
-
-        # performing binary search to locate first line inside clipped area
-        while oldMid != upperMid:
-            oldMid = upperMid
-            [x, y, width, height] = text.getRangeExtents(upperMid,
-                                                         upperMid+1,
-                                                         0)
-            upperY = y
-            if y > cliprect.y:
-                upperMax = upperMid
-            else:
-                upperMin = upperMid
-            upperMid = (upperMax - upperMin) /2 + upperMin
-
-        # performing binary search to locate last line inside clipped area
-        oldMid = 0
-        limit = cliprect.y+cliprect.height
-        while oldMid != lowerMid:
-            oldMid = lowerMid
-            [x, y, width, height] = text.getRangeExtents(lowerMid,
-                                                         lowerMid+1,
-                                                         0)
-            lowerY = y
-            if y > limit:
-                lowerMax = lowerMid
-            else:
-                lowerMin = lowerMid
-            lowerMid = (lowerMax - lowerMin) /2 + lowerMin
-
-        # finding out the zones
-        offset = upperMin
-        length = lowerMax
         while offset < length:
 
             [string, startOffset, endOffset] = text.getTextAtOffset(
